searchData={"items":[{"type":"module","title":"Cldr.Route","doc":"","ref":"Cldr.Route.html"},{"type":"module","title":"Cldr.Routes","doc":"Generate localized route generation, locaverified routes\nand route helper modules.\n\nThis module when `use`d , provides a `localize/1`\nmacro that is designed to wrap the standard Phoenix\nroute macros such as `get/3`, `put/3` and `resources/3`\nand localizes them for each locale defined in a Gettext\nbackend module attached to a Cldr backend module.\n\nTranslations for the parts of a given route path are\ntranslated at compile-time and are then combined into\na localised route that is added to the standard\nPhoenix routing framework.\n\nAs a result, users can enter URLs using localised\nterms which can enhance user engagement and content\nrelevance.\n\nSimilarly, localised path and URL helpers are\ngenerated that wrap the standard Phoenix helpers to\nsupporting generating localised paths and URLs.","ref":"Cldr.Routes.html"},{"type":"module","title":"Configuration - Cldr.Routes","doc":"A Cldr backend module that configures a `Gettext`\nasosciated backend is required.\n\nPath parts (the parts between \"/\") are translated\nat compile time using `Gettext`. Therefore localization\ncan only be applied to locales that are defined in\na [gettext backend module](https://hexdocs.pm/gettext/Gettext.html#module-using-gettext)\nthat is configured in a `Cldr` backend module.\n\nFor example:\n```\ndefmodule MyApp.Cldr do\n  use Cldr,\n    locales: [\"en\", \"fr\"],\n    default_locale: \"en\".\n    gettext: MyApp.Gettext\n    providers: [Cldr.Routes]\n\nend\n```\n\nHere the `MyApp.Cldr` backend module\nis used to instrospect the configured\nlocales in order to drive the localization\ngeneration.\n\nNext, configure the router module to\nuse the `localize/1` macro by adding\n`use MyApp.Cldr.Route` to the module and invoke\nthe `localize/1` macro to wrap the required\nroutes. For example:\n\n```elixir\ndefmodule MyApp.Router do\n  use Phoenix.Router\n  use MyApp.Cldr.Routes\n\n  localize do\n    get \"/pages/:page\", PageController, :show\n    resources \"/users\", UserController\n  end\nend\n```\n\nThe following routes are generated in the `MyApp.Router`\nmodule (assuming that translations are updated in the `Gettext`\nconfiguration).\n\nFor this example, the `:fr` and `:de` translations are\nthe same as the `:en` text with `_fr` or `_de` appended.\n```bash\n% mix phx.routes MyApp.Router\npage_de_path  GET     /pages_de/:page     PageController :show\npage_en_path  GET     /pages/:page        PageController :show\npage_fr_path  GET     /pages_fr/:page     PageController :show\nuser_de_path  GET     /users_de           UserController :index\nuser_de_path  GET     /users_de/:id/edit  UserController :edit\nuser_de_path  GET     /users_de/new       UserController :new\nuser_de_path  GET     /users_de/:id       UserController :show\n...\n```\n\nIt is also possible to emit the routes and route helpers before\nlocalization using the `mix phx.routes` task on an automatically\ngenerated mini-router. See `h MyApp.Router.LocalizedRoutes`.","ref":"Cldr.Routes.html#module-configuration"},{"type":"module","title":"Interpolating Locale Data - Cldr.Routes","doc":"A route may be defined with elements of the locale\ninterpolated into it. These interpolations are specified\nusing the normal `#{}` interpolation syntax. However\nsince route translation occurs at compile time only\nthe following interpolations are supported:\n\n* `locale` will interpolate the Cldr locale name\n* `language` will interpolate the Cldr language name\n* `territory` will interpolate the Cldr territory code\n\nSome examples are:\n```elixir\nlocalize do\n  get \"/#{locale}/locale/pages/:page\", PageController, :show\n  get \"/#{language}/language/pages/:page\", PageController, :show\n  get \"/#{territory}/territory/pages/:page\", PageController, :show\nend\n```","ref":"Cldr.Routes.html#module-interpolating-locale-data"},{"type":"module","title":"Localized Helpers - Cldr.Routes","doc":"Manually constructing the localized helper names shown in\nthe example above would be tedious. Therefore a `LocalizedHelpers`\nmodule is generated at compile-time. Assuming the router\nmodule is called `MyApp.Router` then the full name of the\nlocalized helper module is `MyApp.Router.LocalizedHelpers`.\n\nAs of Phoenix 1.7, a Helpers module is generrated by default but\nthis behaviour can be disabled by adding `helpers: false` to the\n`use Phoenix.Router` call. The same is true of localized helpers.\nThey will be generated by default but can be disabled by adding\n`helpers: false` to the `use MyApp.Cldr.Routes` line in your\nrouter module.\n\nThe functions on this module are the non-localized versions.\nLocalised routes can be printed by leveraging the `phx.routes`\nmix task.  For example:\n```elixir\n% mix phx.routes MyApp.Router\npage_de_path  GET      /pages_de/:page     PageController :show\npage_en_path  GET      /pages/:page        PageController :show\npage_fr_path  GET      /pages_fr/:page     PageController :show\nuser_de_path  GET      /users_de           UserController :index\nuser_de_path  GET      /users_de/:id/edit  UserController :edit\nuser_de_path  GET      /users_de/new       UserController :new\nuser_de_path  GET      /users_de/:id       UserController :show\nuser_de_path  POST     /users_de           UserController :create\nuser_de_path  PATCH    /users_de/:id       UserController :update\n              PUT   /users_de/:id       UserController :update\n.....\n```\n\nThe functions on the `MyApp.Router.LocalizedHelpers` module all\nrespect the current locale, based upon `Cldr.get_locale/1`, and will\ndelegate to the appropriate localized function in the\n`MyApp.Router.Helpers` module created automatically by Phoenix\nat compile time.","ref":"Cldr.Routes.html#module-localized-helpers"},{"type":"module","title":"Configuring Localized Helpers as default - Cldr.Routes","doc":"Since `LocalizedHelpers` have the same semantics and\nAPI as the standard `Helpers` module it is possible to\nupdate the generated Phoenix configuration to use the\n`LocalizedHelpers` module by default.  Assuming the\npresence of `myapp_web.ex` defining the module `MyAppWeb`\nthen changing the `view_helpers` function from\n```elixir\ndefp view_helpers do\n  quote do\n    ...\n    use Gettext, backend: MyApp.Gettext\n    import MyAppWeb.ErrorHelpers\n    alias MyAppWeb.Router.Helpers, as: Routes\n  end\nend\n```\nto\n```elixir\ndefp view_helpers do\n  quote do\n    ...\n\n    use Gettext, backend: MyApp.Gettext\n    import MyAppWeb.ErrorHelpers\n    alias MyAppWeb.Router.LocalizedHelpers, as: Routes\n  end\nend\n```\nwill result in the automatic use of the localized\nhelpers rather than the standard helpers.","ref":"Cldr.Routes.html#module-configuring-localized-helpers-as-default"},{"type":"module","title":"Translations - Cldr.Routes","doc":"In order for routes to be localized, translations must be\nprovided for each path segment. This translation is performed\nby `Gettext.dgettext/3` with the domain \"routes\". Therefore for\neach configured locale, a \"routes.pot\" file is required containing\nthe path segment translations for that locale.\n\nUsing the example Cldr backend that has \"en\" and \"fr\" Gettext\nlocales then the directory structure would look like the following\n(if the default Gettext configuration is used):\n\n    priv/gettext\n    ├── default.pot\n    ├── en\n    │   └── LC_MESSAGES\n    │       ├── default.po\n    │       ├── errors.po\n    │       └── routes.po\n    ├── errors.pot\n    └── fr\n        └── LC_MESSAGES\n            ├── default.po\n            ├── errors.po\n            └── routes.po","ref":"Cldr.Routes.html#module-translations"},{"type":"function","title":"Cldr.Routes.interpolate/2","doc":"","ref":"Cldr.Routes.html#interpolate/2"},{"type":"macro","title":"Cldr.Routes.localize/1","doc":"Generates localised routes for each locale defined in a\nCldr backend.\n\nThis macro is intended to wrap a series of standard\nroute definitiosn in a `do` block. For example:\n\n```elixir\nlocalize do\n  get \"/pages/:page\", PageController, :show\n  resources \"/users\", UsersController\nend\n```","ref":"Cldr.Routes.html#localize/1"},{"type":"macro","title":"Cldr.Routes.localize/2","doc":"Generates localised routes for each locale provided.\n\nThis macro is intended to wrap a series of standard\nroute definitiosn in a `do` block. For example:\n\n```elixir\nlocalize [:en, :fr] do\n  get \"/pages/:page\", PageController, :show\n  resources \"/users\", UsersController\nend\n```","ref":"Cldr.Routes.html#localize/2"},{"type":"function","title":"Cldr.Routes.translate_path_now/3","doc":"","ref":"Cldr.Routes.html#translate_path_now/3"},{"type":"module","title":"Cldr.Routes.LocalizedHelpers","doc":"Generates a module that implements localised helpers.\n\nIt introspects the generated helpers module and creates\na wrapper function that translates (at compile time) the\npath segments.","ref":"Cldr.Routes.LocalizedHelpers.html"},{"type":"function","title":"Cldr.Routes.LocalizedHelpers.define/3","doc":"For a given set of routes, define a LocalizedHelpers\nmodule that implements localized helpers.","ref":"Cldr.Routes.LocalizedHelpers.html#define/3"},{"type":"function","title":"Cldr.Routes.LocalizedHelpers.helper_by_locale/1","doc":"","ref":"Cldr.Routes.LocalizedHelpers.html#helper_by_locale/1"},{"type":"function","title":"Cldr.Routes.LocalizedHelpers.hreflang_links/1","doc":"Generates HTML `link` tags for a given map of locale => URLs\n\nThis function generates ` ` tags that should be placed in the\n` ` section of an HTML document to indicate the different language\nversions of a given page.\n\nThe `MyApp.Router.LocalizedHelpers. _link` functions can\ngenerate the required mapping from locale to URL for a given helper.\nThese `_link` helpers take the same arguments as the `_path` and\n`_url` helpers.\n\nIf the helper refers to a route that is not localized then an\nempty string will be returned since there are no alternative\nlocalizations of this route.\n\nSee https://developers.google.com/search/docs/advanced/crawling/localized-versions#http","ref":"Cldr.Routes.LocalizedHelpers.html#hreflang_links/1"},{"type":"function","title":"Examples - Cldr.Routes.LocalizedHelpers.hreflang_links/1","doc":"iex> links = %{\n    ...>   \"en\" => \"https://localhost/users/1\",\n    ...>   \"fr\" => \"https://localhost/utilisateurs/1\"\n    ...>  }\n    iex> Cldr.Routes.LocalizedHelpers.hreflang_links(links)\n    {\n      :safe,\n      [\n        [60, \"link\", [32, \"href\", 61, 34, \"https://localhost/users/1\", 34, 32, \"hreflang\", 61, 34, \"en\", 34, 32, \"rel\", 61, 34, \"alternate\", 34], 62],\n        10,\n        [60, \"link\", [32, \"href\", 61, 34, \"https://localhost/utilisateurs/1\", 34, 32, \"hreflang\", 61, 34, \"fr\", 34, 32, \"rel\", 61, 34, \"alternate\", 34], 62]\n      ]\n    }\n\n    iex> Cldr.Routes.LocalizedHelpers.hreflang_links(nil)\n    {:safe, []}\n\n    iex> Cldr.Routes.LocalizedHelpers.hreflang_links(%{})\n    {:safe, []}","ref":"Cldr.Routes.LocalizedHelpers.html#hreflang_links/1-examples"},{"type":"type","title":"Cldr.Routes.LocalizedHelpers.locale_name/0","doc":"","ref":"Cldr.Routes.LocalizedHelpers.html#t:locale_name/0"},{"type":"type","title":"Cldr.Routes.LocalizedHelpers.url/0","doc":"","ref":"Cldr.Routes.LocalizedHelpers.html#t:url/0"},{"type":"extras","title":"Cldr Routes","doc":"# Cldr Routes\n![Build Status](http://sweatbox.noexpectations.com.au:8080/buildStatus/icon?job=cldr_routes)\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_cldr_routes.svg)](https://hex.pm/packages/ex_cldr_routes)\n[![Hex.pm](https://img.shields.io/hexpm/dw/ex_cldr_routes.svg?)](https://hex.pm/packages/ex_cldr_routes)\n[![Hex.pm](https://img.shields.io/hexpm/dt/ex_cldr_routes.svg?)](https://hex.pm/packages/ex_cldr_routes)\n[![Hex.pm](https://img.shields.io/hexpm/l/ex_cldr_routes.svg)](https://hex.pm/packages/ex_cldr_routes)\n\nGenerate localized routes and a localized path helper module.\n\nThis module when `use`d , provides a `localize/1` macro that is designed to wrap the standard [Phoenix route](https://hexdocs.pm/phoenix/routing.html) macros such as `get/3`, `put/3` and `resources/3`. The routes are localised for each locale configured in a `Gettext` backend module that is attached to a `Cldr` backend module.\n\nTranslations for the parts of a given route path are performed at compile-time and are then combined into a localised route that is added to the standard Phoenix routing framework.\n\nAs a result, users can enter URLs using localised terms which can enhance user engagement and content relevance.\n\nSimilarly, localised path and URL helpers are generated that wrap the standard [Phoenix helpers](https://hexdocs.pm/phoenix/routing.html#path-helpers) to support generating localised paths and URLs.\n\nLastly, Localized Verified Routes, introduced in Phoenix 1.7, are supported and their use encouraged in preference to URL Helpers. Localized Verified Routes are specified with the `~q` sigil in the same manner as Phoenix Verified Routes `~p`.","ref":"readme.html"},{"type":"extras","title":"Setting up - Cldr Routes","doc":"","ref":"readme.html#setting-up"},{"type":"extras","title":"Installation - Cldr Routes","doc":"The first step is to configure and install by adding `ex_cldr_routes` to your list of dependencies in `mix.exs` then run `mix deps.get`.\n\n```elixir\ndef deps do\n  [\n    {:ex_cldr_routes, \"~> 1.0\"}\n  ]\nend\n```\n\nThe docs can be found at https://hexdocs.pm/ex_cldr_routes.","ref":"readme.html#installation"},{"type":"extras","title":"Backend module configuration - Cldr Routes","doc":"A `Cldr` backend module that configures an associated `gettext` backend is required. In addition, a `Gettext` backend must be configured and added to the `Cldr` configuration.\n\nPath parts (the parts between \"/\") are translated at compile time using `Gettext`. Therefore localization can only be applied to locales that are defined in a [gettext backend module](https://hexdocs.pm/gettext/Gettext.html#module-using-gettext) that is attached to a `Cldr` backend module.\n\nThe following steps should be followed to set up the configuration for localized routes and helpers:","ref":"readme.html#backend-module-configuration"},{"type":"extras","title":"Configure Gettext - Cldr Routes","doc":"The first step is to ensure there is a configured `Gettext` backend module:\n\n```elixir\ndefmodule MyApp.Gettext do\n  use Gettext.Backend, otp_app: :my_app\nend\n```","ref":"readme.html#configure-gettext"},{"type":"extras","title":"Configure Cldr - Cldr Routes","doc":"The next step is to configure a `Cldr` backend module, including configuring it with the `Gettext` module defined in the first step. The `MyApp.Cldr` backend module is used to instrospect the configured locales that drive the route and helper localization.\n\n```elixir\ndefmodule MyApp.Cldr do\n  use Cldr,\n    locales: [\"en\", \"fr\"],\n    default_locale: \"en\",\n    gettext: MyApp.Gettext,\n    providers: [Cldr.Routes]\n\nend\n```\n\n*Note* the addition of `Cldr.Routes` to the `:providers` configuration key is required.","ref":"readme.html#configure-cldr"},{"type":"extras","title":"Define Localized Routes - Cldr Routes","doc":"Now we can configure the router module to use the `localize/1` macro by adding `use MyApp.Cldr.Routes` to the module and invoke the `localize/1` macro to wrap the required routes. `use MyApp.Cldr.Routes` must be added *after* `use Phoenix.Router`. For example:\n\n```elixir\ndefmodule MyAppWeb.Router do\n  use Phoenix.Router\n  use MyApp.Cldr.Routes\n\n  localize do\n    get \"/pages/:page\", PageController, :show\n    resources \"/users\", UserController\n  end\nend\n```\n\nThe following routes are generated (assuming that translations are updated in the `Gettext` configuration). For this example, the `:fr` translations are the same as the `:en` text with `_fr` appended.\n```bash\n% mix phx.routes MyAppWeb.Router\npage_de_path  GET     /pages_de/:page     PageController :show\npage_en_path  GET     /pages/:page        PageController :show\npage_fr_path  GET     /pages_fr/:page     PageController :show\nuser_de_path  GET     /users_de           UserController :index\nuser_de_path  GET     /users_de/:id/edit  UserController :edit\nuser_de_path  GET     /users_de/new       UserController :new\nuser_de_path  GET     /users_de/:id       UserController :show\n...\n```","ref":"readme.html#define-localized-routes"},{"type":"extras","title":"Interpolating Locale Data - Cldr Routes","doc":"A route may be defined with elements of the locale interpolated into it. These interpolatins are specified using the normal `#{}` interpolation syntax. However since route translation occurs at compile time only the following interpolations are supported:\n\n* `locale` will interpolate the Cldr locale name\n* `language` will interpolate the Cldr language name\n* `territory` will interpolate the Cldr territory code\n\nSome examples are:\n```elixir\nlocalize do\n  get \"/#{locale}/locale/pages/:page\", PageController, :show\n  get \"/#{language}/language/pages/:page\", PageController, :show\n  get \"/#{territory}/territory/pages/:page\", PageController, :show\nend\n```","ref":"readme.html#interpolating-locale-data"},{"type":"extras","title":"Localized Helpers - Cldr Routes","doc":"> #### Warning {: .warning}\n>\n> Route helpers are deprecated in favor of verified routes as of Phoenix 1.7. Like Phoenix,\n> localization route helpers can still be generated. When using `mix phx.new my_app`,\n> the `MyAppWeb` module will include `use Phoenix.Routes, helpers: false`. This is also the\n> recommendation when applying `use MyApp.Cldr.Routes`.\n\nManually constructing the localized helper names shown in the example above would be tedious. Therefore a `LocalizedHelpers` module is generated at compile-time. Assuming the router module is called `MyAppWeb.Router` then the full name of the localized helper module is `MyAppWeb.Router.LocalizedHelpers`.\n\nThe functions on this module are the non-localized versions that should be used by applications (they delegate ultimately to the localized routes based upon the current locale).\n\nThe functions on the `LocalizedHelpers` module all respect the current locale, based upon `Cldr.get_locale/1`, and will delegate to the appropriate localized function in the `Helpers` function created automatically at compile time. For example:\n```elixir\niex> MyApp.Cldr.put_locale(\"en\")\niex> MyAppWeb.Router.LocalizedHelpers.page_path %Plug.Conn{}, :show, 1\n\"/pages/1\"\niex> MyApp.Cldr.put_locale(\"fr\")\niex> MyAppWeb.Router.LocalizedHelpers.page_path %Plug.Conn{}, :show, 1\n\"/pages_fr/1\"\n```\n\n*Note* The localized helpers translate the path based upon the `:gettext_locale_name` for the currently set `Cldr` locale. It is the developers responsibility to ensure that the locale is set appropriately before calling any localized path helpers.","ref":"readme.html#localized-helpers"},{"type":"extras","title":"Introspecting localized routes - Cldr Routes","doc":"For convenience in introspecting routes, a module called `MyApp.Router.LocalizedRoutes` is generated that can be used with the `mix phx.routes` mix task. For example:\n\n```bash\n% cldr_routes % mix phx.routes MyAppWeb.Router.LocalizedRoutes\npage_path  GET     /pages_de/:page     PageController :show\npage_path  GET     /pages/:page        PageController :show\npage_path  GET     /pages_fr/:page     PageController :show\nuser_path  GET     /users_de           UserController :index\nuser_path  GET     /users_de/:id/edit  UserController :edit\nuser_path  GET     /users_de/new       UserController :new\n...\n```\n\nIn addition, each localized path stores the `Cldr` locale in the `:private` field for the route under the `:cldr_locale` key. This allows the developer to recognise which locale was used to generate the localized route.\n\nThis information is also used by functions in the [ex_cldr_plugs](https://hex.pm/packages/ex_cldr_plugs) library to:\n\n* Identify the users locale from the route in `Cldr.Plug.PutLocale`\n* Store the identified locale in the session in `Cldr.Plug.PutSession`\n* Propogate the locale from the session into a LiveView process during the `on_mount/3` callback with `Cldr.Plug.put_locale_from_session/2`","ref":"readme.html#introspecting-localized-routes"},{"type":"extras","title":"Localized Verified Routes - Cldr Routes","doc":"`Sigil_q` implements localized verified routes for Phoenix 1.7 and later.\n\nAdding\n```elixir\nuse MyApp.Cldr.VerifiedRoutes,\n  router: MyApp.Router,\n  endpoint: MyApp.Endpoint\n```\nto a module gives access to `sigil_q` which is functionally equal to Phoenix Verified Routes `sigil_p`. In fact the result of using `sigil_q` is code that looks like this:\n\n```elixir\n# ~q\"/users\" generates the following code for a\n# Cldr backend that has configured the locales\n# :en, :fr and :de\n\ncase MyApp.Cldr.get_locale().cldr_locale_name do\n  :de -> ~p\"/users_de\"\n  :en -> ~p\"/users\"\n  :fr -> ~p\"/users_fr\"\nend\n```","ref":"readme.html#localized-verified-routes"},{"type":"extras","title":"Locale interpolation - Cldr Routes","doc":"Some use cases call for the locale, language or territory to be part of the URL. `Sigl_q` makes this easy by providing\nthe following interpolations into a `Sigil_q` path:\n\n* `:locale` is replaced with Cldr locale name.\n* `:language` is replaced with the Cldr language code.\n* `:territory` is replaced with the Cldr territory code.\n\n```elixir\n# ~q\"/users/:locale\" generates the following code for a\n# Cldr backend that has configured the locales\n# :en, :fr and :de. Note the interpolation of the locale\n# information which is performed at compile time.\n\ncase MyApp.Cldr.get_locale().cldr_locale_name do\n  :de -> ~p\"/users_de/de\"\n  :en -> ~p\"/users/en\"\n  :fr -> ~p\"/users_fr/fr\"\nend\n```","ref":"readme.html#locale-interpolation"},{"type":"extras","title":"Phoenix MyWebApp configuration - Cldr Routes","doc":"Since localized routes, route helpers and verified routes have the same function and API as the standard Phoenix equivalent modules it is possible to update the generated Phoenix configuration. Assuming the presence of `myapp_web.ex` defining the module `MyAppWeb` then the following changes should be considered in the `my_web_app.ex` file:","ref":"readme.html#phoenix-mywebapp-configuration"},{"type":"extras","title":"Router - Cldr Routes","doc":"```elixir\ndef router do\n  quote do\n    use Phoenix.Router, helpers: false\n\n    # Add localized routes\n    use MyApp.Cldr.Routes, helpers: false\n\n    # Import common connection and controller functions to use in pipelines\n    import Plug.Conn\n    import Phoenix.Controller\n    import Phoenix.LiveView.Router\n  end\nend\n```","ref":"readme.html#router"},{"type":"extras","title":"Verified Routes - Cldr Routes","doc":"Change `Phoenix.VerifiedRoutes` to `MyApp.Cldr.VerifiedRoutes`:\n```elixir\n  def verified_routes do\n    quote do\n      use MyApp.Cldr.VerifiedRoutes,\n        endpoint: PhxCldrWeb.Endpoint,\n        router: PhxCldrWeb.Router,\n        statics: PhxCldrWeb.static_paths()\n    end\n  end\n```","ref":"readme.html#verified-routes"},{"type":"extras","title":"Generating link headers - Cldr Routes","doc":"When the same content is produced in multiple languages it is important to cross-link the different editions of the content to each other. This is good practise in general but strong advised by [goggle](https://developers.google.com/search/docs/advanced/crawling/localized-versions) to reduce SEO risks for your site.\n\nThis cross-linking can be done with the aid of HTTP headers or with ` ` tags in the ` ` section of an HTML document. Helpers are generated by `ex_cldr_routes` to facilitate the creating of these links. These functions are generated in the `MyAppWeb.Router.LocalizedHelpers` module (where `MyAppWeb.Router` is the name of your Phoenix router module).\n\n* `MyAppWeb.Router.LocalizedHelpers. _links` generated a mapping of locales for a given route to the URLs for those locales.\n* `MyAppWeb.Router.LocalizedHelpers.hreflang_links/1` take the generated map and produces link headers ready for insertion into an HTML document.\n\nThe functions can be used as follows:\n\n1. Update the controller to generate the links and add them to `:assigns`\n\n```elixir\ndefmodule MyAppWeb.UserController do\n  use MyAppWeb, :controller\n\n  # This alias would normally be set in the\n  # MyAppWeb.controller/0 and MyAppWeb.view_helpers/0\n  # functions\n  alias MyAppWeb.Router.LocalizedHelpers, as: Routes\n\n  alias MyApp.Accounts\n  alias MyApp.Accounts.User\n\n  def show(conn, %{\"id\" => id}) do\n    user = Accounts.get_user!(id)\n    hreflang_links = Routes.user_links(conn, :show, id)\n    render(conn, \"show.html\", user: user, hreflang_links: hreflang_links)\n  end\nend\n```\n\n2. Update the root layout to add the hreflang links\n\n```html\n \n   \n     \n     \n     \n    <%= Routes.hreflang_links(assigns[:hreflang_links]) %>\n    <%= csrf_meta_tag() %>\n    <%= live_title_tag assigns[:page_title] || \"Routing\", suffix: \" · Phoenix Framework\" %>\n     \n      \n   \n   \n    ...\n    <%= @inner_content %>\n   \n \n```","ref":"readme.html#generating-link-headers"},{"type":"extras","title":"Translations - Cldr Routes","doc":"In order for routes to be localized, translations must be provided for each path segment. This translation is performed by `dgettext/3` with the domain \"routes\". Therefore for each configured locale, a \"routes.pot\" file is required containing the path segment translations for that locale.\n\nUsing the example Cldr backend that has \"en\" and \"fr\" Gettext locales then the directory structure would look like the following (if the default Gettext configuration is used):\n\n    priv/gettext\n    ├── default.pot\n    ├── en\n    │   └── LC_MESSAGES\n    │       ├── default.po\n    │       ├── errors.po\n    │       └── routes.po\n    ├── errors.pot\n    └── fr\n        └── LC_MESSAGES\n            ├── default.po\n            ├── errors.po\n            └── routes.po\n\nThe `mix` tasks `gettext.extract` and `gettext.merge` can be used to support the extraction of routing segments and to create new translation locales.","ref":"readme.html#translations"},{"type":"extras","title":"License","doc":"# License\n\nCopyright 2018-2020 Kip Cole\n\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use this file except in\ncompliance with the License. You may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software distributed under the License\nis distributed on an \"AS IS\" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\nimplied. See the License for the specific language governing permissions and limitations under the\nLicense.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Cldr Routes v1.3.1 - Changelog","doc":"This is the changelog for Cldr Routes version 1.3.1 released on August 23rd, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v1-3-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Requires [Gettext version 0.26](https://hex.pm/packages/gettext/0.26.0) or later since that version has an incompatible API with previsous versions and we wish to avoid more conditional code. Thanks to @krns for the PR to align the documentation and examples as well.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr Routes v1.3.0 - Changelog","doc":"This is the changelog for Cldr Routes version 1.3.0 released on February 19th, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v1-3-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds support for calling `url/{1,2,3}` with a `sigil_q` (localized verified route) parameter. The `Phoenix.VerifiedRoutes.url/1` macro checks that its parameter is a `sigil_p` call. Since `sigil_q` returns a `case` expression with one or more case clauses returning a `sigil_p` result, the `Phoenix.VerifiedRoutes.url/1` macro cannot be used directly. This release adds `url/{1,2,3}` to the `MyApp.Cldr.VerifiedRoutes` module so it is compatible with existing code and supports both `sigil_p` and `sigil_q` parameters. Thanks to @rigzad for the issue. Closes #16.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr Routes v1.2.0 - Changelog","doc":"This is the changelog for Cldr Routes version 1.2.0 released on January 3rd, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v1-2-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Configures `:phoenix_html_helpers` as a dependency rather than the full `:phoenix` app. `:ex_cldr_routes` only uses the tag helper which is now hosted in the new library. Whilst deprecated within Phoenix, its use here is to generate `hreflang` headers.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr Routes v1.1.0 - Changelog","doc":"This is the changelog for Cldr Routes version 1.1.0 released on May 9th, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v1-1-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix Phoenix integration which was failing to compile when using Phoenix auth. Thanks to @rubas for the report and collaboration.\n\n* Fix dialyzer error. Thanks to @rubas for the report.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecation - Changelog","doc":"* The module `Cldr.Route` is renamed to `Cldr.Routes` to better match Phoenix naming.  As a result, the provider module to be added to a Cldr backend configuration is now `Cldr.Routes`, not `Cldr.Route`.  `Cldr.Route` remains for now and if used will issue a deprecation warning.","ref":"changelog.html#deprecation"},{"type":"extras","title":"Cldr Routes v1.0.0 - Changelog","doc":"This is the changelog for Cldr Routes version 1.0.0 released on May 3rd, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v1-0-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Supports localized verified routes with `~q` (`Sigil_q`).\n\n* Supports Phoenix 1.7 and later only.\n\n* Supports Elixir 1.11 and later only.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr Routes v0.6.4 - Changelog","doc":"This is the changelog for Cldr Routes version 0.6.4 released on April 29th, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v0-6-4"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes route `:as` option to correctly stringify atom names. Closes #14. Thanks to @krns for the report.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr Routes v0.6.3 - Changelog","doc":"This is the changelog for Cldr Routes version 0.6.2 released on April 27th, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v0-6-3"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix readme example. Thanks to @krns for the PR. Closes #12.\n\n* Support Phoenix 1.7.  NOTE: Doesn't yet include localized verified routes.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr Routes v0.6.2 - Changelog","doc":"This is the changelog for Cldr Routes version 0.6.2 released on August 6th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v0-6-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Uses the standard Phoenix `tag` helper to generate `hreflang` helpers.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr Routes v0.6.1 - Changelog","doc":"This is the changelog for Cldr Routes version 0.6.1 released on July 24th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v0-6-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix `LocalizedHelpers.hreflang_links/1` to return an empty string if links is `nil`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr Routes v0.6.0 - Changelog","doc":"This is the changelog for Cldr Routes version 0.6.0 released on July 24th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v0-6-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds `MyApp.Router.LocalizedHelpers. _links` functions to the generated `LocalizedHelpers` module. These `_links` functions are 1:1 correspondents to the `_path` and `_url` helpers. The `_link` helpers generate link headers that help identify the other language versions of a page. They are used like this:\n```elixir\niex> alias MyApp.Router.LocalizedHelpers, as: Routes\niex> Routes.user_links(conn, :show, 1) |> Routes.hreflang_links()\n{\n :safe,\n [\n   [\" \"],\n   \"\\n\",\n   [\" \"],\n   \"\\n\",\n   [\" \"]\n  ]\n}\n```","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr Routes v0.5.0 - Changelog","doc":"This is the changelog for Cldr Routes version 0.5.0 released on July 22nd, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v0-5-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Propogate locales on the `localize` macro to nested resources","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking change - Changelog","doc":"* The locale is now stored in the `:private` field of the `conn` for both live routes and other routes. It was previously stored in the `:assigns` field for non-live routes.","ref":"changelog.html#breaking-change"},{"type":"extras","title":"Cldr Routes v0.4.0 - Changelog","doc":"This is the changelog for Cldr Routes version 0.4.0 released on July 19th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v0-4-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixed Localized route helpers were matching on the full locale, not on the `:gettext_locale_name` field. Thanks to @rubas for the report and collaboration. Closes #6.\n\n* `mix phx.routes MyApp.Router.LocalizedRoutes` was attempting to \"un\"-translate the routes. This is no longer the case since doing so hides information required by developers. Closes #8.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr Routes v0.3.0 - Changelog","doc":"This is the changelog for Cldr Routes version 0.3.0 released on July 17th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v0-3-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add support for `live` routes. Thanks to @ringofhealth for the report. Closes #1.\n\n* Support interpolating `locale`, `language` and `territory` into a route. Thanks to @rubas for the suggestion. Closes #3. For example:\n\n```elixir\n  localize do\n    get \"/#{locale}/locale/pages/:page\", PageController, :show, as: \"with_locale\"\n    get \"/#{language}/language/pages/:page\", PageController, :show, as: \"with_language\"\n    get \"/#{territory}/territory/pages/:page\", PageController, :show, as: \"with_territory\"\n  end\n```\n\n* Uses the macros from Gettext rather than the functions when generating routes. This means that the mix tasks `gettext.extract` and `gettext.merge` now work as expected. Closes #3.\n\n* Rewrite the `LocalizedHelpers` module that now proxies into the standard Phoenix `Helpers` module rather than maintaining a separate forked module.  As a result:\n  * The standard `Helpers` module now generates helper names that have locale suffixes.  That is `user_path` becomes `user_en_path`, `user_fr_path` and so on.\n  * The `LocalizedHelpers` module hosts the standard helper names (like `user_path`) which will then call the appropriate standard helper depending on the result of `Cldr.get_locale/1`.\n\n* Add functions to output the localised routes. At compile time a module called `MyApp.Router.LocalizedRoutes` is created.  This module hosts a `__routes__/0` function which can be passed as an argument to the Phoenix standard ` Phoenix.Router.ConsoleFormatter.format/1` function that returns a string representation of configured localized routes. These can then be `IO.puts/1` as required.  In the next release a mix task will automate this process.\n\nThanks to @rubas and @ringofhealth for their extreme patience while I worked this through. Closes #1, and #4.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr Routes v0.2.0 - Changelog","doc":"This is the changelog for Cldr Routes version 0.2.0 released on March 26th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v0-2-0"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* Changes the module name from `Cldr.Routes` to `Cldr.Route` to be consistent with the other `ex_cldr`-based libraries which use singular module names.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Cldr Routes v0.1.0 - Changelog","doc":"This is the changelog for Cldr Routes version 0.1.0 released on March 26th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_routes/tags)","ref":"changelog.html#cldr-routes-v0-1-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Initial release","ref":"changelog.html#enhancements"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}